==> rakuten_spiders-0.0.1-py3-none-any.whl: rakuten/spiders/rakuten_specs_spider.py <==
from datetime import datetime
import csv
import json

import scrapy
import w3lib.html
from urllib.parse import urlparse


from resources.base_spider import BaseSpider


class RakutenSpecsSpiderSpider(BaseSpider):
    name = "rakuten_specs_spider"
    cookies = {
        "Rp": "05917b7ae4ad75d2cdbcd61856661aee265945",
        "_ra": "1719245896500|f9e9ad5a-3cbe-4e8a-b005-b3f009258fb9",
        "ak_bmsc": "7C6C63D1FDF52745F7AF851F5A50F4FA~000000000000000000000000000000~YAAQRLx3aCWrwhCQAQAA8X8KSxh44Bg5XUGH9F4T+E7ix5RIutx6sFUhGhcZWcndzuGdbji5Uvrcxepx3UcL7SdxCZlIVT90YEz302yAG2eExfIFXjh60W3DcHO/Dhp2lDCJKtKQQQRk4ByEgZp1yVmtqeW9nqdw+Y0tv8QZwoBDcPl6PRt9EdRcWQYDOyFAJPWlX+UNtnboYYmpsllRbT1B58/eXG2a75Pay3lquc2HzRBpiN3CAN2RK1YhSHvGnwhnW0QZmR+N0Q6rMAEd2hYwQRYxZxNsc7iVdCTEskY63eY7oVvWiXowL/iVYNmN+0c2t7HBaPM/ILQngnLQQTGBC91kr5D3JyFHrSSvfJX8w9VpZ0dr4mV7AdIBXEVml15VxPhVhizlos7KGgBmHvKjJQGq0irpg6NK0Jxd2BD/mXEnwgQdwlLUZPFLPk2QW67Juws=",
        "__lt__cid.3df24f5b": "bbcba912-773c-4f0d-b884-3bbe80651a63",
        "_uetsid": "6303c220324511ef94b18f865bfc8636",
        "_uetvid": "6303d830324511efbaaab713d51cdb8b",
        "_yjsu_yjad": "1719245906.5f3a6a6f-edce-44d8-aa91-b1e0a14f5138",
        "_gcl_au": "1.1.1451697634.1719245907",
        "cto_bundle": "pcz8rV9RZm8lMkYzSnBNNWNhekx3YW1KckxvejFoTHkzb2NUeXJkczhrTVd3anhwampmdkk4M0FpTHJOVnNkczUwZTRtelV1RFA3T3V5NGxzbU9qT1ZxenVnR0c2RG1DTmhWMiUyRkg4VGRlJTJCUWNlb3d6VHVZRmlyd3ZRT3RXVDYydktCVFhnVExoRE1LeDFRJTJCNVJMJTJCVXJ1NE4yVllGS3NVTjdqdXd0VVZPTm5wcXdkTzdwNmdPRWdxQlRTa0hYU2x4WU5zbVBIT0V5T2NlWEtneVlIYVJEd2VmRkhaNzYxSUx3YzJmU01adnAlMkZ0Q2htWURWdWR2TVFHRVRvZUZ5ZmM0YjlLTmhlc05KZGNwM3pTVFJZcWVaQVdvczJxVUZNU0NQUzdjamVVdkNySU1ZUVU2b3BpMDBoSVh3NmR2MWtSTTFSVlN0Zg",
        "_clck": "1gi16or%7C2%7Cfmw%7C0%7C1636",
        "_clsk": "kjdt74%7C1719245908257%7C1%7C0%7Cv.clarity.ms%2Fcollect",
        "sui_hide_shipping": "20230316",
        "Apache": "c503d5b7.61ba58f50f0ed",
        "bm_mi": "D58C5A7CEF5A06ACAB4F2F2DC238EF8E~YAAQrUg/FzoabQeQAQAA8rwmSxjtQMMu0lSEjZAOxSXRmXBp2X2deMMhRfci8A9YslGE4Np57sLas5YtCih71+WXMo2tapcXT2bKFJvn1I4jY5uTjJDGl+BoSr39gXxEr4CIUurUx/Uf+Rb1Hbx0Oe6zist7Xh5D7NdYolJg0Io9Q1F3JdL6/a3hCk8G+V8oyfqEq9q2HFWILNSYRlQasfogMCs7bvum12NHIAtC/wmlTPGIRRFHiYDMSGw0u/gqOBqd67mb6LvtOBFcWxeIoBllBiL9TZUPWdyK074CM0CPsYSPtQEuALD3DYY/LrxfjsK4WLHwi6MlNV54hkljxwIC+gE22U7KEO1yYxBftw==~1",
        "bm_sv": "CCE87BCD6A3BA73C9D2AF011C4809736~YAAQrUg/FzsabQeQAQAA8rwmSxjJUmdmioC3kJABc5icoujiZnqQ3DXr641gSK8NFxG3K9RWZNJvuUIOFFYSplCRNjeMcsiPhoeTPaRQDoRry2NFxFxEQ4fqYoDwxz86nSvdMek4YNttrh4JaNLfg5AYN7k85REwLLH/03c9mCT8QVcg3NtJf4o4SQcJF+JRxmfiFPnBiQos8iNE+P9dlwOxUFiE4fN2XhEN43QxCOW+nm+rBFvpdnE6cBBYgzYKARuX~1",
    }

    headers = {
        "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
        "accept-language": "en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7",
        "cache-control": "no-cache",
        "pragma": "no-cache",
        "priority": "u=0, i",
        "sec-ch-ua": '"Chromium";v="124", "Google Chrome";v="124", "Not-A.Brand";v="99"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Linux"',
        "sec-fetch-dest": "document",
        "sec-fetch-mode": "navigate",
        "sec-fetch-site": "none",
        "sec-fetch-user": "?1",
        "upgrade-insecure-requests": "1",
        "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36",
    }

    custom_settings = {
        # "DOWNLOADER_CLIENT_TLS_CIPHERS": BaseSpider.ssl(),
        "ITEM_PIPELINES": {
            "rakuten.pipelines.RakutenPipeline": 300,
        },
        "DOWNLOADER_MIDDLEWARES": {
            #     "scrapy.contrib.downloadermiddleware.useragent.UserAgentMiddleware": None,
            #     "scrapy_random_ua.RandomUserAgentMiddleware": 400,
            # "scrapy_webshare.middleware.WebshareMiddleware": 500,
        },
    }

    start_urls = []

    def is_valid_url(self, url):
        try:
            result = urlparse(url)
            return all([result.scheme, result.netloc])
        except ValueError:
            return False

    def start_requests(self):
        for doc in self.start_urls:
            if not self.is_valid_url(doc.get("url")):
                continue

            yield scrapy.Request(doc["url"], headers=self.headers, meta={"_id": doc["_id"]})

    def is_blank(self, value):
        blanks = ["various numbers", "ノーブランド", "various countries", "なし"]
        for blank in blanks:
            if blank in value:
                return True

    def get_specifications(self, json_data):
        try:
            specs = []
            attrs = json_data["data"]["itemInfoSku"]["attributes"]
            for attr in attrs:
                if self.is_blank(attr["value"]):
                    continue

                spec = {}
                spec["name"] = attr["title"]
                spec["value"] = attr["value"]
                specs.append(spec)

            return None if not specs else specs
        except Exception as e:
            pass

    def parse(self, response):
        self.logger.info(f"[url]: {response.url}")
        id = response.meta["_id"]
        data = response.css("script::text").re_first('"api": (.+),')
        specifications = None
        if data:
            try:
                json_data = json.loads(data)
                specifications = self.get_specifications(json_data)
            except Exception as e:
                pass
        yield {
            "_id": id,
            "specifications": specifications,
            "time": datetime.now().replace(microsecond=0),
        }

==> rakuten_spiders-0.0.6-py3-none-any.whl: rakuten/spiders/rakuten_specs_spider.py <==
from datetime import datetime
import csv
import json

import scrapy
import w3lib.html
from urllib.parse import urlparse


from em_product.resources.base_spider import BaseSpider


class RakutenSpecsSpiderSpider(BaseSpider):
    name = "rakuten_specs_spider"
    cookies = {
        "Rp": "05917b7ae4ad75d2cdbcd61856661aee265945",
        "_ra": "1719245896500|f9e9ad5a-3cbe-4e8a-b005-b3f009258fb9",
        "ak_bmsc": "7C6C63D1FDF52745F7AF851F5A50F4FA~000000000000000000000000000000~YAAQRLx3aCWrwhCQAQAA8X8KSxh44Bg5XUGH9F4T+E7ix5RIutx6sFUhGhcZWcndzuGdbji5Uvrcxepx3UcL7SdxCZlIVT90YEz302yAG2eExfIFXjh60W3DcHO/Dhp2lDCJKtKQQQRk4ByEgZp1yVmtqeW9nqdw+Y0tv8QZwoBDcPl6PRt9EdRcWQYDOyFAJPWlX+UNtnboYYmpsllRbT1B58/eXG2a75Pay3lquc2HzRBpiN3CAN2RK1YhSHvGnwhnW0QZmR+N0Q6rMAEd2hYwQRYxZxNsc7iVdCTEskY63eY7oVvWiXowL/iVYNmN+0c2t7HBaPM/ILQngnLQQTGBC91kr5D3JyFHrSSvfJX8w9VpZ0dr4mV7AdIBXEVml15VxPhVhizlos7KGgBmHvKjJQGq0irpg6NK0Jxd2BD/mXEnwgQdwlLUZPFLPk2QW67Juws=",
        "__lt__cid.3df24f5b": "bbcba912-773c-4f0d-b884-3bbe80651a63",
        "_uetsid": "6303c220324511ef94b18f865bfc8636",
        "_uetvid": "6303d830324511efbaaab713d51cdb8b",
        "_yjsu_yjad": "1719245906.5f3a6a6f-edce-44d8-aa91-b1e0a14f5138",
        "_gcl_au": "1.1.1451697634.1719245907",
        "cto_bundle": "pcz8rV9RZm8lMkYzSnBNNWNhekx3YW1KckxvejFoTHkzb2NUeXJkczhrTVd3anhwampmdkk4M0FpTHJOVnNkczUwZTRtelV1RFA3T3V5NGxzbU9qT1ZxenVnR0c2RG1DTmhWMiUyRkg4VGRlJTJCUWNlb3d6VHVZRmlyd3ZRT3RXVDYydktCVFhnVExoRE1LeDFRJTJCNVJMJTJCVXJ1NE4yVllGS3NVTjdqdXd0VVZPTm5wcXdkTzdwNmdPRWdxQlRTa0hYU2x4WU5zbVBIT0V5T2NlWEtneVlIYVJEd2VmRkhaNzYxSUx3YzJmU01adnAlMkZ0Q2htWURWdWR2TVFHRVRvZUZ5ZmM0YjlLTmhlc05KZGNwM3pTVFJZcWVaQVdvczJxVUZNU0NQUzdjamVVdkNySU1ZUVU2b3BpMDBoSVh3NmR2MWtSTTFSVlN0Zg",
        "_clck": "1gi16or%7C2%7Cfmw%7C0%7C1636",
        "_clsk": "kjdt74%7C1719245908257%7C1%7C0%7Cv.clarity.ms%2Fcollect",
        "sui_hide_shipping": "20230316",
        "Apache": "c503d5b7.61ba58f50f0ed",
        "bm_mi": "D58C5A7CEF5A06ACAB4F2F2DC238EF8E~YAAQrUg/FzoabQeQAQAA8rwmSxjtQMMu0lSEjZAOxSXRmXBp2X2deMMhRfci8A9YslGE4Np57sLas5YtCih71+WXMo2tapcXT2bKFJvn1I4jY5uTjJDGl+BoSr39gXxEr4CIUurUx/Uf+Rb1Hbx0Oe6zist7Xh5D7NdYolJg0Io9Q1F3JdL6/a3hCk8G+V8oyfqEq9q2HFWILNSYRlQasfogMCs7bvum12NHIAtC/wmlTPGIRRFHiYDMSGw0u/gqOBqd67mb6LvtOBFcWxeIoBllBiL9TZUPWdyK074CM0CPsYSPtQEuALD3DYY/LrxfjsK4WLHwi6MlNV54hkljxwIC+gE22U7KEO1yYxBftw==~1",
        "bm_sv": "CCE87BCD6A3BA73C9D2AF011C4809736~YAAQrUg/FzsabQeQAQAA8rwmSxjJUmdmioC3kJABc5icoujiZnqQ3DXr641gSK8NFxG3K9RWZNJvuUIOFFYSplCRNjeMcsiPhoeTPaRQDoRry2NFxFxEQ4fqYoDwxz86nSvdMek4YNttrh4JaNLfg5AYN7k85REwLLH/03c9mCT8QVcg3NtJf4o4SQcJF+JRxmfiFPnBiQos8iNE+P9dlwOxUFiE4fN2XhEN43QxCOW+nm+rBFvpdnE6cBBYgzYKARuX~1",
    }

    headers = {
        "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
        "accept-language": "en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7",
        "cache-control": "no-cache",
        "pragma": "no-cache",
        "priority": "u=0, i",
        "sec-ch-ua": '"Chromium";v="124", "Google Chrome";v="124", "Not-A.Brand";v="99"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Linux"',
        "sec-fetch-dest": "document",
        "sec-fetch-mode": "navigate",
        "sec-fetch-site": "none",
        "sec-fetch-user": "?1",
        "upgrade-insecure-requests": "1",
        "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36",
    }

    custom_settings = {
        # "DOWNLOADER_CLIENT_TLS_CIPHERS": BaseSpider.ssl(),
        "ITEM_PIPELINES": {
            "rakuten.pipelines.RakutenPipeline": 300,
        },
        "DOWNLOADER_MIDDLEWARES": {
            #     "scrapy.contrib.downloadermiddleware.useragent.UserAgentMiddleware": None,
            #     "scrapy_random_ua.RandomUserAgentMiddleware": 400,
            # "scrapy_webshare.middleware.WebshareMiddleware": 500,
        },
    }

    start_urls = []

    def is_valid_url(self, url):
        try:
            result = urlparse(url)
            return all([result.scheme, result.netloc])
        except ValueError:
            return False

    def start_requests(self):
        for doc in self.start_urls:
            if not self.is_valid_url(doc.get("url")):
                continue

            yield scrapy.Request(doc["url"], headers=self.headers, meta={"_id": doc["_id"]})

    def is_blank(self, value):
        blanks = ["various numbers", "ノーブランド", "various countries", "なし"]
        for blank in blanks:
            if blank in value:
                return True

    def get_specifications(self, json_data):
        try:
            specs = []
            attrs = json_data["data"]["itemInfoSku"]["attributes"]
            for attr in attrs:
                if self.is_blank(attr["value"]):
                    continue

                spec = {}
                spec["name"] = attr["title"]
                spec["value"] = attr["value"]
                specs.append(spec)

            return None if not specs else specs
        except Exception as e:
            pass

    def parse(self, response):
        self.logger.info(f"[url]: {response.url}")
        id = response.meta["_id"]
        data = response.css("script::text").re_first('"api": (.+),')
        specifications = None
        if data:
            try:
                json_data = json.loads(data)
                specifications = self.get_specifications(json_data)
            except Exception as e:
                pass
        yield {
            "_id": id,
            "specifications": specifications,
            "time": datetime.now().replace(microsecond=0),
        }
